# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: storage.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import bookkeeper.proto.common_pb2 as common__pb2
import bookkeeper.proto.stream_pb2 as stream__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='storage.proto',
  package='bookkeeper.proto.storage',
  syntax='proto3',
  serialized_options=_b('\n*org.apache.bookkeeper.stream.proto.storageP\001'),
  serialized_pb=_b('\n\rstorage.proto\x12\x18\x62ookkeeper.proto.storage\x1a\x0c\x63ommon.proto\x1a\x0cstream.proto\"l\n\x16GetActiveRangesRequest\x12\x11\n\tstream_id\x18\x01 \x01(\x03\x12?\n\x0cstream_props\x18\x02 \x01(\x0b\x32).bookkeeper.proto.stream.StreamProperties\"\x86\x01\n\x17GetActiveRangesResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12\x37\n\x06ranges\x18\x02 \x03(\x0b\x32\'.bookkeeper.proto.storage.RelatedRanges\"\x96\x01\n\rRelatedRanges\x12\x37\n\x05props\x18\x01 \x01(\x0b\x32(.bookkeeper.proto.stream.RangeProperties\x12\x34\n\x04type\x18\x02 \x01(\x0e\x32&.bookkeeper.proto.storage.RelationType\x12\x16\n\x0erelated_ranges\x18\x03 \x03(\x03\"h\n\x16\x43reateNamespaceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12@\n\x07ns_conf\x18\x02 \x01(\x0b\x32/.bookkeeper.proto.stream.NamespaceConfiguration\"\x8d\x01\n\x17\x43reateNamespaceResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12>\n\x08ns_props\x18\x02 \x01(\x0b\x32,.bookkeeper.proto.stream.NamespaceProperties\"&\n\x16\x44\x65leteNamespaceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"M\n\x17\x44\x65leteNamespaceResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\"#\n\x13GetNamespaceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x8a\x01\n\x14GetNamespaceResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12>\n\x08ns_props\x18\x02 \x01(\x0b\x32,.bookkeeper.proto.stream.NamespaceProperties\"w\n\x13\x43reateStreamRequest\x12\x0f\n\x07ns_name\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x41\n\x0bstream_conf\x18\x03 \x01(\x0b\x32,.bookkeeper.proto.stream.StreamConfiguration\"\x8b\x01\n\x14\x43reateStreamResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12?\n\x0cstream_props\x18\x02 \x01(\x0b\x32).bookkeeper.proto.stream.StreamProperties\"4\n\x13\x44\x65leteStreamRequest\x12\x0f\n\x07ns_name\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"J\n\x14\x44\x65leteStreamResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\"i\n\x10GetStreamRequest\x12:\n\x0bstream_name\x18\x01 \x01(\x0b\x32#.bookkeeper.proto.stream.StreamNameH\x00\x12\x13\n\tstream_id\x18\x02 \x01(\x03H\x00\x42\x04\n\x02id\"\x88\x01\n\x11GetStreamResponse\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12?\n\x0cstream_props\x18\x02 \x01(\x0b\x32).bookkeeper.proto.stream.StreamProperties\"\xba\x01\n\x18StorageContainerEndpoint\x12\x1c\n\x14storage_container_id\x18\x01 \x01(\x03\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x36\n\x0brw_endpoint\x18\x03 \x01(\x0b\x32!.bookkeeper.proto.common.Endpoint\x12\x36\n\x0bro_endpoint\x18\x04 \x03(\x0b\x32!.bookkeeper.proto.common.Endpoint\"Q\n\"OneStorageContainerEndpointRequest\x12\x19\n\x11storage_container\x18\x01 \x01(\x03\x12\x10\n\x08revision\x18\x02 \x01(\x03\"t\n\"GetStorageContainerEndpointRequest\x12N\n\x08requests\x18\x01 \x03(\x0b\x32<.bookkeeper.proto.storage.OneStorageContainerEndpointRequest\"\xa6\x01\n#OneStorageContainerEndpointResponse\x12\x39\n\x0bstatus_code\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12\x44\n\x08\x65ndpoint\x18\x02 \x01(\x0b\x32\x32.bookkeeper.proto.storage.StorageContainerEndpoint\"\xb2\x01\n#GetStorageContainerEndpointResponse\x12\x39\n\x0bstatus_code\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12P\n\tresponses\x18\x02 \x03(\x0b\x32=.bookkeeper.proto.storage.OneStorageContainerEndpointResponse*\xca\x04\n\nStatusCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x10\n\x0b\x42\x41\x44_REQUEST\x10\x90\x03\x12\x0f\n\nILLEGAL_OP\x10\x93\x03\x12\x1a\n\x15INTERNAL_SERVER_ERROR\x10\xf4\x03\x12\x14\n\x0fNOT_IMPLEMENTED\x10\xf5\x03\x12\x0f\n\nUNEXPECTED\x10\xd8\x04\x12\x10\n\x0b\x42\x41\x44_VERSION\x10\x84\x07\x12\x11\n\x0c\x42\x41\x44_REVISION\x10\x85\x07\x12\x1b\n\x16INVALID_NAMESPACE_NAME\x10\xd0\x0f\x12\x15\n\x10NAMESPACE_EXISTS\x10\xd1\x0f\x12\x18\n\x13NAMESPACE_NOT_FOUND\x10\xd2\x0f\x12\x18\n\x13INVALID_STREAM_NAME\x10\xb4\x10\x12\x12\n\rSTREAM_EXISTS\x10\xb5\x10\x12\x15\n\x10STREAM_NOT_FOUND\x10\xb6\x10\x12 \n\x1bINVALID_STREAMSNAPSHOT_NAME\x10\xb7\x10\x12\x1a\n\x15STREAMSNAPSHOT_EXISTS\x10\xb8\x10\x12\x1d\n\x18STREAMSNAPSHOT_NOT_FOUND\x10\xb9\x10\x12\x19\n\x14WRONG_GROUP_LOCATION\x10\xb8\x17\x12\x15\n\x10INVALID_GROUP_ID\x10\xb9\x17\x12\x11\n\x0cGROUP_EXISTS\x10\xba\x17\x12\x14\n\x0fGROUP_NOT_FOUND\x10\xbb\x17\x12\x15\n\x10STALE_GROUP_INFO\x10\xbc\x17\x12\x10\n\x0bINVALID_KEY\x10\xf0.\x12\x0f\n\nKEY_EXISTS\x10\xf1.\x12\x12\n\rKEY_NOT_FOUND\x10\xf2.*?\n\x0cRelationType\x12\x0c\n\x08\x43HILDREN\x10\x00\x12\x0b\n\x07PARENTS\x10\x01\x12\x14\n\x10\x43HILDREN_PARENTS\x10\x02\x32\x8a\x01\n\x10MetaRangeService\x12v\n\x0fGetActiveRanges\x12\x30.bookkeeper.proto.storage.GetActiveRangesRequest\x1a\x31.bookkeeper.proto.storage.GetActiveRangesResponse2\xb5\x05\n\x10RootRangeService\x12v\n\x0f\x43reateNamespace\x12\x30.bookkeeper.proto.storage.CreateNamespaceRequest\x1a\x31.bookkeeper.proto.storage.CreateNamespaceResponse\x12v\n\x0f\x44\x65leteNamespace\x12\x30.bookkeeper.proto.storage.DeleteNamespaceRequest\x1a\x31.bookkeeper.proto.storage.DeleteNamespaceResponse\x12m\n\x0cGetNamespace\x12-.bookkeeper.proto.storage.GetNamespaceRequest\x1a..bookkeeper.proto.storage.GetNamespaceResponse\x12m\n\x0c\x43reateStream\x12-.bookkeeper.proto.storage.CreateStreamRequest\x1a..bookkeeper.proto.storage.CreateStreamResponse\x12m\n\x0c\x44\x65leteStream\x12-.bookkeeper.proto.storage.DeleteStreamRequest\x1a..bookkeeper.proto.storage.DeleteStreamResponse\x12\x64\n\tGetStream\x12*.bookkeeper.proto.storage.GetStreamRequest\x1a+.bookkeeper.proto.storage.GetStreamResponse2\xb6\x01\n\x17StorageContainerService\x12\x9a\x01\n\x1bGetStorageContainerEndpoint\x12<.bookkeeper.proto.storage.GetStorageContainerEndpointRequest\x1a=.bookkeeper.proto.storage.GetStorageContainerEndpointResponseB.\n*org.apache.bookkeeper.stream.proto.storageP\x01\x62\x06proto3')
  ,
  dependencies=[common__pb2.DESCRIPTOR,stream__pb2.DESCRIPTOR,])

_STATUSCODE = _descriptor.EnumDescriptor(
  name='StatusCode',
  full_name='bookkeeper.proto.storage.StatusCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BAD_REQUEST', index=2, number=400,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ILLEGAL_OP', index=3, number=403,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SERVER_ERROR', index=4, number=500,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_IMPLEMENTED', index=5, number=501,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNEXPECTED', index=6, number=600,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BAD_VERSION', index=7, number=900,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BAD_REVISION', index=8, number=901,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_NAMESPACE_NAME', index=9, number=2000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NAMESPACE_EXISTS', index=10, number=2001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NAMESPACE_NOT_FOUND', index=11, number=2002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_STREAM_NAME', index=12, number=2100,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STREAM_EXISTS', index=13, number=2101,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STREAM_NOT_FOUND', index=14, number=2102,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_STREAMSNAPSHOT_NAME', index=15, number=2103,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STREAMSNAPSHOT_EXISTS', index=16, number=2104,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STREAMSNAPSHOT_NOT_FOUND', index=17, number=2105,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WRONG_GROUP_LOCATION', index=18, number=3000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_GROUP_ID', index=19, number=3001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GROUP_EXISTS', index=20, number=3002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GROUP_NOT_FOUND', index=21, number=3003,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STALE_GROUP_INFO', index=22, number=3004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_KEY', index=23, number=6000,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KEY_EXISTS', index=24, number=6001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KEY_NOT_FOUND', index=25, number=6002,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2398,
  serialized_end=2984,
)
_sym_db.RegisterEnumDescriptor(_STATUSCODE)

StatusCode = enum_type_wrapper.EnumTypeWrapper(_STATUSCODE)
_RELATIONTYPE = _descriptor.EnumDescriptor(
  name='RelationType',
  full_name='bookkeeper.proto.storage.RelationType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CHILDREN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARENTS', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHILDREN_PARENTS', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2986,
  serialized_end=3049,
)
_sym_db.RegisterEnumDescriptor(_RELATIONTYPE)

RelationType = enum_type_wrapper.EnumTypeWrapper(_RELATIONTYPE)
SUCCESS = 0
FAILURE = 1
BAD_REQUEST = 400
ILLEGAL_OP = 403
INTERNAL_SERVER_ERROR = 500
NOT_IMPLEMENTED = 501
UNEXPECTED = 600
BAD_VERSION = 900
BAD_REVISION = 901
INVALID_NAMESPACE_NAME = 2000
NAMESPACE_EXISTS = 2001
NAMESPACE_NOT_FOUND = 2002
INVALID_STREAM_NAME = 2100
STREAM_EXISTS = 2101
STREAM_NOT_FOUND = 2102
INVALID_STREAMSNAPSHOT_NAME = 2103
STREAMSNAPSHOT_EXISTS = 2104
STREAMSNAPSHOT_NOT_FOUND = 2105
WRONG_GROUP_LOCATION = 3000
INVALID_GROUP_ID = 3001
GROUP_EXISTS = 3002
GROUP_NOT_FOUND = 3003
STALE_GROUP_INFO = 3004
INVALID_KEY = 6000
KEY_EXISTS = 6001
KEY_NOT_FOUND = 6002
CHILDREN = 0
PARENTS = 1
CHILDREN_PARENTS = 2



_GETACTIVERANGESREQUEST = _descriptor.Descriptor(
  name='GetActiveRangesRequest',
  full_name='bookkeeper.proto.storage.GetActiveRangesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream_id', full_name='bookkeeper.proto.storage.GetActiveRangesRequest.stream_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_props', full_name='bookkeeper.proto.storage.GetActiveRangesRequest.stream_props', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=179,
)


_GETACTIVERANGESRESPONSE = _descriptor.Descriptor(
  name='GetActiveRangesResponse',
  full_name='bookkeeper.proto.storage.GetActiveRangesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.GetActiveRangesResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ranges', full_name='bookkeeper.proto.storage.GetActiveRangesResponse.ranges', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=182,
  serialized_end=316,
)


_RELATEDRANGES = _descriptor.Descriptor(
  name='RelatedRanges',
  full_name='bookkeeper.proto.storage.RelatedRanges',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='props', full_name='bookkeeper.proto.storage.RelatedRanges.props', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='bookkeeper.proto.storage.RelatedRanges.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='related_ranges', full_name='bookkeeper.proto.storage.RelatedRanges.related_ranges', index=2,
      number=3, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=319,
  serialized_end=469,
)


_CREATENAMESPACEREQUEST = _descriptor.Descriptor(
  name='CreateNamespaceRequest',
  full_name='bookkeeper.proto.storage.CreateNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bookkeeper.proto.storage.CreateNamespaceRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ns_conf', full_name='bookkeeper.proto.storage.CreateNamespaceRequest.ns_conf', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=471,
  serialized_end=575,
)


_CREATENAMESPACERESPONSE = _descriptor.Descriptor(
  name='CreateNamespaceResponse',
  full_name='bookkeeper.proto.storage.CreateNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.CreateNamespaceResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ns_props', full_name='bookkeeper.proto.storage.CreateNamespaceResponse.ns_props', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=578,
  serialized_end=719,
)


_DELETENAMESPACEREQUEST = _descriptor.Descriptor(
  name='DeleteNamespaceRequest',
  full_name='bookkeeper.proto.storage.DeleteNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bookkeeper.proto.storage.DeleteNamespaceRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=721,
  serialized_end=759,
)


_DELETENAMESPACERESPONSE = _descriptor.Descriptor(
  name='DeleteNamespaceResponse',
  full_name='bookkeeper.proto.storage.DeleteNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.DeleteNamespaceResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=761,
  serialized_end=838,
)


_GETNAMESPACEREQUEST = _descriptor.Descriptor(
  name='GetNamespaceRequest',
  full_name='bookkeeper.proto.storage.GetNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bookkeeper.proto.storage.GetNamespaceRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=840,
  serialized_end=875,
)


_GETNAMESPACERESPONSE = _descriptor.Descriptor(
  name='GetNamespaceResponse',
  full_name='bookkeeper.proto.storage.GetNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.GetNamespaceResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ns_props', full_name='bookkeeper.proto.storage.GetNamespaceResponse.ns_props', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=878,
  serialized_end=1016,
)


_CREATESTREAMREQUEST = _descriptor.Descriptor(
  name='CreateStreamRequest',
  full_name='bookkeeper.proto.storage.CreateStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ns_name', full_name='bookkeeper.proto.storage.CreateStreamRequest.ns_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='bookkeeper.proto.storage.CreateStreamRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_conf', full_name='bookkeeper.proto.storage.CreateStreamRequest.stream_conf', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1018,
  serialized_end=1137,
)


_CREATESTREAMRESPONSE = _descriptor.Descriptor(
  name='CreateStreamResponse',
  full_name='bookkeeper.proto.storage.CreateStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.CreateStreamResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_props', full_name='bookkeeper.proto.storage.CreateStreamResponse.stream_props', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1140,
  serialized_end=1279,
)


_DELETESTREAMREQUEST = _descriptor.Descriptor(
  name='DeleteStreamRequest',
  full_name='bookkeeper.proto.storage.DeleteStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ns_name', full_name='bookkeeper.proto.storage.DeleteStreamRequest.ns_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='bookkeeper.proto.storage.DeleteStreamRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1281,
  serialized_end=1333,
)


_DELETESTREAMRESPONSE = _descriptor.Descriptor(
  name='DeleteStreamResponse',
  full_name='bookkeeper.proto.storage.DeleteStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.DeleteStreamResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1335,
  serialized_end=1409,
)


_GETSTREAMREQUEST = _descriptor.Descriptor(
  name='GetStreamRequest',
  full_name='bookkeeper.proto.storage.GetStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream_name', full_name='bookkeeper.proto.storage.GetStreamRequest.stream_name', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_id', full_name='bookkeeper.proto.storage.GetStreamRequest.stream_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='id', full_name='bookkeeper.proto.storage.GetStreamRequest.id',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1411,
  serialized_end=1516,
)


_GETSTREAMRESPONSE = _descriptor.Descriptor(
  name='GetStreamResponse',
  full_name='bookkeeper.proto.storage.GetStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.storage.GetStreamResponse.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_props', full_name='bookkeeper.proto.storage.GetStreamResponse.stream_props', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1519,
  serialized_end=1655,
)


_STORAGECONTAINERENDPOINT = _descriptor.Descriptor(
  name='StorageContainerEndpoint',
  full_name='bookkeeper.proto.storage.StorageContainerEndpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storage_container_id', full_name='bookkeeper.proto.storage.StorageContainerEndpoint.storage_container_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='bookkeeper.proto.storage.StorageContainerEndpoint.revision', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rw_endpoint', full_name='bookkeeper.proto.storage.StorageContainerEndpoint.rw_endpoint', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ro_endpoint', full_name='bookkeeper.proto.storage.StorageContainerEndpoint.ro_endpoint', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1658,
  serialized_end=1844,
)


_ONESTORAGECONTAINERENDPOINTREQUEST = _descriptor.Descriptor(
  name='OneStorageContainerEndpointRequest',
  full_name='bookkeeper.proto.storage.OneStorageContainerEndpointRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storage_container', full_name='bookkeeper.proto.storage.OneStorageContainerEndpointRequest.storage_container', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='bookkeeper.proto.storage.OneStorageContainerEndpointRequest.revision', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1846,
  serialized_end=1927,
)


_GETSTORAGECONTAINERENDPOINTREQUEST = _descriptor.Descriptor(
  name='GetStorageContainerEndpointRequest',
  full_name='bookkeeper.proto.storage.GetStorageContainerEndpointRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='requests', full_name='bookkeeper.proto.storage.GetStorageContainerEndpointRequest.requests', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1929,
  serialized_end=2045,
)


_ONESTORAGECONTAINERENDPOINTRESPONSE = _descriptor.Descriptor(
  name='OneStorageContainerEndpointResponse',
  full_name='bookkeeper.proto.storage.OneStorageContainerEndpointResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status_code', full_name='bookkeeper.proto.storage.OneStorageContainerEndpointResponse.status_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='bookkeeper.proto.storage.OneStorageContainerEndpointResponse.endpoint', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2048,
  serialized_end=2214,
)


_GETSTORAGECONTAINERENDPOINTRESPONSE = _descriptor.Descriptor(
  name='GetStorageContainerEndpointResponse',
  full_name='bookkeeper.proto.storage.GetStorageContainerEndpointResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status_code', full_name='bookkeeper.proto.storage.GetStorageContainerEndpointResponse.status_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='responses', full_name='bookkeeper.proto.storage.GetStorageContainerEndpointResponse.responses', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2217,
  serialized_end=2395,
)

_GETACTIVERANGESREQUEST.fields_by_name['stream_props'].message_type = stream__pb2._STREAMPROPERTIES
_GETACTIVERANGESRESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_GETACTIVERANGESRESPONSE.fields_by_name['ranges'].message_type = _RELATEDRANGES
_RELATEDRANGES.fields_by_name['props'].message_type = stream__pb2._RANGEPROPERTIES
_RELATEDRANGES.fields_by_name['type'].enum_type = _RELATIONTYPE
_CREATENAMESPACEREQUEST.fields_by_name['ns_conf'].message_type = stream__pb2._NAMESPACECONFIGURATION
_CREATENAMESPACERESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_CREATENAMESPACERESPONSE.fields_by_name['ns_props'].message_type = stream__pb2._NAMESPACEPROPERTIES
_DELETENAMESPACERESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_GETNAMESPACERESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_GETNAMESPACERESPONSE.fields_by_name['ns_props'].message_type = stream__pb2._NAMESPACEPROPERTIES
_CREATESTREAMREQUEST.fields_by_name['stream_conf'].message_type = stream__pb2._STREAMCONFIGURATION
_CREATESTREAMRESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_CREATESTREAMRESPONSE.fields_by_name['stream_props'].message_type = stream__pb2._STREAMPROPERTIES
_DELETESTREAMRESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_GETSTREAMREQUEST.fields_by_name['stream_name'].message_type = stream__pb2._STREAMNAME
_GETSTREAMREQUEST.oneofs_by_name['id'].fields.append(
  _GETSTREAMREQUEST.fields_by_name['stream_name'])
_GETSTREAMREQUEST.fields_by_name['stream_name'].containing_oneof = _GETSTREAMREQUEST.oneofs_by_name['id']
_GETSTREAMREQUEST.oneofs_by_name['id'].fields.append(
  _GETSTREAMREQUEST.fields_by_name['stream_id'])
_GETSTREAMREQUEST.fields_by_name['stream_id'].containing_oneof = _GETSTREAMREQUEST.oneofs_by_name['id']
_GETSTREAMRESPONSE.fields_by_name['code'].enum_type = _STATUSCODE
_GETSTREAMRESPONSE.fields_by_name['stream_props'].message_type = stream__pb2._STREAMPROPERTIES
_STORAGECONTAINERENDPOINT.fields_by_name['rw_endpoint'].message_type = common__pb2._ENDPOINT
_STORAGECONTAINERENDPOINT.fields_by_name['ro_endpoint'].message_type = common__pb2._ENDPOINT
_GETSTORAGECONTAINERENDPOINTREQUEST.fields_by_name['requests'].message_type = _ONESTORAGECONTAINERENDPOINTREQUEST
_ONESTORAGECONTAINERENDPOINTRESPONSE.fields_by_name['status_code'].enum_type = _STATUSCODE
_ONESTORAGECONTAINERENDPOINTRESPONSE.fields_by_name['endpoint'].message_type = _STORAGECONTAINERENDPOINT
_GETSTORAGECONTAINERENDPOINTRESPONSE.fields_by_name['status_code'].enum_type = _STATUSCODE
_GETSTORAGECONTAINERENDPOINTRESPONSE.fields_by_name['responses'].message_type = _ONESTORAGECONTAINERENDPOINTRESPONSE
DESCRIPTOR.message_types_by_name['GetActiveRangesRequest'] = _GETACTIVERANGESREQUEST
DESCRIPTOR.message_types_by_name['GetActiveRangesResponse'] = _GETACTIVERANGESRESPONSE
DESCRIPTOR.message_types_by_name['RelatedRanges'] = _RELATEDRANGES
DESCRIPTOR.message_types_by_name['CreateNamespaceRequest'] = _CREATENAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['CreateNamespaceResponse'] = _CREATENAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['DeleteNamespaceRequest'] = _DELETENAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['DeleteNamespaceResponse'] = _DELETENAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['GetNamespaceRequest'] = _GETNAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['GetNamespaceResponse'] = _GETNAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['CreateStreamRequest'] = _CREATESTREAMREQUEST
DESCRIPTOR.message_types_by_name['CreateStreamResponse'] = _CREATESTREAMRESPONSE
DESCRIPTOR.message_types_by_name['DeleteStreamRequest'] = _DELETESTREAMREQUEST
DESCRIPTOR.message_types_by_name['DeleteStreamResponse'] = _DELETESTREAMRESPONSE
DESCRIPTOR.message_types_by_name['GetStreamRequest'] = _GETSTREAMREQUEST
DESCRIPTOR.message_types_by_name['GetStreamResponse'] = _GETSTREAMRESPONSE
DESCRIPTOR.message_types_by_name['StorageContainerEndpoint'] = _STORAGECONTAINERENDPOINT
DESCRIPTOR.message_types_by_name['OneStorageContainerEndpointRequest'] = _ONESTORAGECONTAINERENDPOINTREQUEST
DESCRIPTOR.message_types_by_name['GetStorageContainerEndpointRequest'] = _GETSTORAGECONTAINERENDPOINTREQUEST
DESCRIPTOR.message_types_by_name['OneStorageContainerEndpointResponse'] = _ONESTORAGECONTAINERENDPOINTRESPONSE
DESCRIPTOR.message_types_by_name['GetStorageContainerEndpointResponse'] = _GETSTORAGECONTAINERENDPOINTRESPONSE
DESCRIPTOR.enum_types_by_name['StatusCode'] = _STATUSCODE
DESCRIPTOR.enum_types_by_name['RelationType'] = _RELATIONTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetActiveRangesRequest = _reflection.GeneratedProtocolMessageType('GetActiveRangesRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETACTIVERANGESREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetActiveRangesRequest)
  ))
_sym_db.RegisterMessage(GetActiveRangesRequest)

GetActiveRangesResponse = _reflection.GeneratedProtocolMessageType('GetActiveRangesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETACTIVERANGESRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetActiveRangesResponse)
  ))
_sym_db.RegisterMessage(GetActiveRangesResponse)

RelatedRanges = _reflection.GeneratedProtocolMessageType('RelatedRanges', (_message.Message,), dict(
  DESCRIPTOR = _RELATEDRANGES,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.RelatedRanges)
  ))
_sym_db.RegisterMessage(RelatedRanges)

CreateNamespaceRequest = _reflection.GeneratedProtocolMessageType('CreateNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _CREATENAMESPACEREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.CreateNamespaceRequest)
  ))
_sym_db.RegisterMessage(CreateNamespaceRequest)

CreateNamespaceResponse = _reflection.GeneratedProtocolMessageType('CreateNamespaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _CREATENAMESPACERESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.CreateNamespaceResponse)
  ))
_sym_db.RegisterMessage(CreateNamespaceResponse)

DeleteNamespaceRequest = _reflection.GeneratedProtocolMessageType('DeleteNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETENAMESPACEREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.DeleteNamespaceRequest)
  ))
_sym_db.RegisterMessage(DeleteNamespaceRequest)

DeleteNamespaceResponse = _reflection.GeneratedProtocolMessageType('DeleteNamespaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELETENAMESPACERESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.DeleteNamespaceResponse)
  ))
_sym_db.RegisterMessage(DeleteNamespaceResponse)

GetNamespaceRequest = _reflection.GeneratedProtocolMessageType('GetNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACEREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetNamespaceRequest)
  ))
_sym_db.RegisterMessage(GetNamespaceRequest)

GetNamespaceResponse = _reflection.GeneratedProtocolMessageType('GetNamespaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACERESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetNamespaceResponse)
  ))
_sym_db.RegisterMessage(GetNamespaceResponse)

CreateStreamRequest = _reflection.GeneratedProtocolMessageType('CreateStreamRequest', (_message.Message,), dict(
  DESCRIPTOR = _CREATESTREAMREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.CreateStreamRequest)
  ))
_sym_db.RegisterMessage(CreateStreamRequest)

CreateStreamResponse = _reflection.GeneratedProtocolMessageType('CreateStreamResponse', (_message.Message,), dict(
  DESCRIPTOR = _CREATESTREAMRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.CreateStreamResponse)
  ))
_sym_db.RegisterMessage(CreateStreamResponse)

DeleteStreamRequest = _reflection.GeneratedProtocolMessageType('DeleteStreamRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETESTREAMREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.DeleteStreamRequest)
  ))
_sym_db.RegisterMessage(DeleteStreamRequest)

DeleteStreamResponse = _reflection.GeneratedProtocolMessageType('DeleteStreamResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELETESTREAMRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.DeleteStreamResponse)
  ))
_sym_db.RegisterMessage(DeleteStreamResponse)

GetStreamRequest = _reflection.GeneratedProtocolMessageType('GetStreamRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETSTREAMREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetStreamRequest)
  ))
_sym_db.RegisterMessage(GetStreamRequest)

GetStreamResponse = _reflection.GeneratedProtocolMessageType('GetStreamResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETSTREAMRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetStreamResponse)
  ))
_sym_db.RegisterMessage(GetStreamResponse)

StorageContainerEndpoint = _reflection.GeneratedProtocolMessageType('StorageContainerEndpoint', (_message.Message,), dict(
  DESCRIPTOR = _STORAGECONTAINERENDPOINT,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.StorageContainerEndpoint)
  ))
_sym_db.RegisterMessage(StorageContainerEndpoint)

OneStorageContainerEndpointRequest = _reflection.GeneratedProtocolMessageType('OneStorageContainerEndpointRequest', (_message.Message,), dict(
  DESCRIPTOR = _ONESTORAGECONTAINERENDPOINTREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.OneStorageContainerEndpointRequest)
  ))
_sym_db.RegisterMessage(OneStorageContainerEndpointRequest)

GetStorageContainerEndpointRequest = _reflection.GeneratedProtocolMessageType('GetStorageContainerEndpointRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETSTORAGECONTAINERENDPOINTREQUEST,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetStorageContainerEndpointRequest)
  ))
_sym_db.RegisterMessage(GetStorageContainerEndpointRequest)

OneStorageContainerEndpointResponse = _reflection.GeneratedProtocolMessageType('OneStorageContainerEndpointResponse', (_message.Message,), dict(
  DESCRIPTOR = _ONESTORAGECONTAINERENDPOINTRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.OneStorageContainerEndpointResponse)
  ))
_sym_db.RegisterMessage(OneStorageContainerEndpointResponse)

GetStorageContainerEndpointResponse = _reflection.GeneratedProtocolMessageType('GetStorageContainerEndpointResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETSTORAGECONTAINERENDPOINTRESPONSE,
  __module__ = 'storage_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.storage.GetStorageContainerEndpointResponse)
  ))
_sym_db.RegisterMessage(GetStorageContainerEndpointResponse)


DESCRIPTOR._options = None

_METARANGESERVICE = _descriptor.ServiceDescriptor(
  name='MetaRangeService',
  full_name='bookkeeper.proto.storage.MetaRangeService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=3052,
  serialized_end=3190,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetActiveRanges',
    full_name='bookkeeper.proto.storage.MetaRangeService.GetActiveRanges',
    index=0,
    containing_service=None,
    input_type=_GETACTIVERANGESREQUEST,
    output_type=_GETACTIVERANGESRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_METARANGESERVICE)

DESCRIPTOR.services_by_name['MetaRangeService'] = _METARANGESERVICE


_ROOTRANGESERVICE = _descriptor.ServiceDescriptor(
  name='RootRangeService',
  full_name='bookkeeper.proto.storage.RootRangeService',
  file=DESCRIPTOR,
  index=1,
  serialized_options=None,
  serialized_start=3193,
  serialized_end=3886,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateNamespace',
    full_name='bookkeeper.proto.storage.RootRangeService.CreateNamespace',
    index=0,
    containing_service=None,
    input_type=_CREATENAMESPACEREQUEST,
    output_type=_CREATENAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteNamespace',
    full_name='bookkeeper.proto.storage.RootRangeService.DeleteNamespace',
    index=1,
    containing_service=None,
    input_type=_DELETENAMESPACEREQUEST,
    output_type=_DELETENAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetNamespace',
    full_name='bookkeeper.proto.storage.RootRangeService.GetNamespace',
    index=2,
    containing_service=None,
    input_type=_GETNAMESPACEREQUEST,
    output_type=_GETNAMESPACERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateStream',
    full_name='bookkeeper.proto.storage.RootRangeService.CreateStream',
    index=3,
    containing_service=None,
    input_type=_CREATESTREAMREQUEST,
    output_type=_CREATESTREAMRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteStream',
    full_name='bookkeeper.proto.storage.RootRangeService.DeleteStream',
    index=4,
    containing_service=None,
    input_type=_DELETESTREAMREQUEST,
    output_type=_DELETESTREAMRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetStream',
    full_name='bookkeeper.proto.storage.RootRangeService.GetStream',
    index=5,
    containing_service=None,
    input_type=_GETSTREAMREQUEST,
    output_type=_GETSTREAMRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ROOTRANGESERVICE)

DESCRIPTOR.services_by_name['RootRangeService'] = _ROOTRANGESERVICE


_STORAGECONTAINERSERVICE = _descriptor.ServiceDescriptor(
  name='StorageContainerService',
  full_name='bookkeeper.proto.storage.StorageContainerService',
  file=DESCRIPTOR,
  index=2,
  serialized_options=None,
  serialized_start=3889,
  serialized_end=4071,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetStorageContainerEndpoint',
    full_name='bookkeeper.proto.storage.StorageContainerService.GetStorageContainerEndpoint',
    index=0,
    containing_service=None,
    input_type=_GETSTORAGECONTAINERENDPOINTREQUEST,
    output_type=_GETSTORAGECONTAINERENDPOINTRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STORAGECONTAINERSERVICE)

DESCRIPTOR.services_by_name['StorageContainerService'] = _STORAGECONTAINERSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

__PYTHON3__ = sys.version_info >= (3, 0)


def to_bytes(n, length, endianess='big'):
    if __PYTHON3__:
        return n.to_bytes(length, endianess)
    else:
        h = '%x' % n
        s = ('0'*(len(h) % 2) + h).zfill(length*2).decode('hex')
        return s if endianess == 'big' else s[::-1]


def new_hostname_with_port(hostname, default_port=4181):
    host_parts = hostname.split(':')
    if len(host_parts) > 1:
        return hostname
    else:
        return "%s:%d" % (hostname, default_port)



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import grpc
import logging

from bookkeeper import types
from bookkeeper.admin.namespace import Namespace
from bookkeeper.admin.namespaces import Namespaces
from bookkeeper.common.service_uri import ServiceURI
from bookkeeper.proto.storage_pb2_grpc import RootRangeServiceStub

__logger__ = logging.getLogger("bookkeeper.admin.Client")


class Client(object):
    """An admin client for Apache BookKeeper.

    This creates an object that is capable of administrating bookkeeper
    resources. Generally, you can instantiate this client with no arguments,
    and you get sensible defaults.

    Args:
        storage_client_settings (~bookkeeper.types.StorageClientSettings): The
            settings for bookkeeper storage client .
        kwargs (dict): Any additional arguments provided are sent as keyword
            arguments to the underlying grpc client.
    """

    def __init__(self, storage_client_settings=(), **kwargs):
        # init the storage client settings
        self.storage_client_settings =\
            types.StorageClientSettings(*storage_client_settings)
        __logger__.info("Creating an admin client to cluster '%s'",
                        self.storage_client_settings.service_uri)

        service_uri = ServiceURI(self.storage_client_settings.service_uri)
        assert service_uri.service_name.lower() == 'bk'

        # create channel
        self.__channel__ = grpc.insecure_channel(
            target=service_uri.service_location
        )
        __logger__.info("Successfully created an admin client to cluster '%s'",
                        self.storage_client_settings.service_uri)

        # create the rpc stub
        self.__root_range__ = RootRangeServiceStub(channel=self.__channel__)

        # services
        self.__namespaces__ = Namespaces(self.__root_range__)

    def namespaces(self):
        return self.__namespaces__

    def namespace(self, namespace):
        return Namespace(self.__root_range__, namespace)



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from bookkeeper.common.constants import __DEFAULT_NS_CONF__
from bookkeeper.common.constants import __ROOT_RANGE_METADATA__
from bookkeeper.common.exceptions import from_root_range_rpc_response
from bookkeeper.common.exceptions import InternalServerError
from bookkeeper.common.exceptions import NamespaceExistsError
from bookkeeper.common.exceptions import NamespaceNotFoundError
from bookkeeper.common.method import wrap_method
from bookkeeper.common.retry import Retry
from bookkeeper.common.timeout import ExponentialTimeout
from bookkeeper.proto import storage_pb2


class Namespaces(object):

    def __init__(self, root_range_service):
        self.__root_range_service__ = root_range_service
        self.__default_retry__ = Retry(deadline=60)
        self.__default_timeout__ = ExponentialTimeout()
        self.__create_with_retries__ =\
            wrap_method(self.__create_ns__, self.__default_retry__)
        self.__get_with_retries__ =\
            wrap_method(self.__get_ns__, self.__default_retry__)
        self.__delete_with_retries__ =\
            wrap_method(self.__delete_ns__, self.__default_retry__)

    def create(self, namespace, namespace_config=__DEFAULT_NS_CONF__):
        return self.__create_with_retries__(namespace, namespace_config)

    def __create_ns__(self, namespace, namespace_config):
        create_ns_req = storage_pb2.CreateNamespaceRequest(
            name=namespace,
            ns_conf=namespace_config
        )
        create_ns_resp = self.__root_range_service__.CreateNamespace(
            request=create_ns_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        try:
            create_ns_resp = from_root_range_rpc_response(create_ns_resp)
            return create_ns_resp.ns_props
        except InternalServerError as ise:
            # currently if a namespace exists, it also throws
            # internal server error.
            try:
                self.get(namespace=namespace)
                raise NamespaceExistsError("namespace '%s' already exists" % namespace)
            except NamespaceNotFoundError:
                raise ise

    def get(self, namespace):
        return self.__get_with_retries__(namespace)

    def __get_ns__(self, namespace):
        get_ns_req = storage_pb2.GetNamespaceRequest(
            name=namespace
        )
        get_ns_resp = self.__root_range_service__.GetNamespace(
            request=get_ns_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        get_ns_resp = from_root_range_rpc_response(get_ns_resp)
        return get_ns_resp.ns_props

    def delete(self, namespace):
        return self.__delete_with_retries__(namespace)

    def __delete_ns__(self, namespace):
        del_ns_req = storage_pb2.DeleteNamespaceRequest(
            name=namespace
        )
        del_ns_resp = self.__root_range_service__.DeleteNamespace(
            request=del_ns_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        from_root_range_rpc_response(del_ns_resp)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import bookkeeper.proto.storage_pb2 as storage__pb2


class MetaRangeServiceStub(object):
  """public service for other operations in range server
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetActiveRanges = channel.unary_unary(
        '/bookkeeper.proto.storage.MetaRangeService/GetActiveRanges',
        request_serializer=storage__pb2.GetActiveRangesRequest.SerializeToString,
        response_deserializer=storage__pb2.GetActiveRangesResponse.FromString,
        )


class MetaRangeServiceServicer(object):
  """public service for other operations in range server
  """

  def GetActiveRanges(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_MetaRangeServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetActiveRanges': grpc.unary_unary_rpc_method_handler(
          servicer.GetActiveRanges,
          request_deserializer=storage__pb2.GetActiveRangesRequest.FromString,
          response_serializer=storage__pb2.GetActiveRangesResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bookkeeper.proto.storage.MetaRangeService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class RootRangeServiceStub(object):
  """public service for metadata services
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateNamespace = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/CreateNamespace',
        request_serializer=storage__pb2.CreateNamespaceRequest.SerializeToString,
        response_deserializer=storage__pb2.CreateNamespaceResponse.FromString,
        )
    self.DeleteNamespace = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/DeleteNamespace',
        request_serializer=storage__pb2.DeleteNamespaceRequest.SerializeToString,
        response_deserializer=storage__pb2.DeleteNamespaceResponse.FromString,
        )
    self.GetNamespace = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/GetNamespace',
        request_serializer=storage__pb2.GetNamespaceRequest.SerializeToString,
        response_deserializer=storage__pb2.GetNamespaceResponse.FromString,
        )
    self.CreateStream = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/CreateStream',
        request_serializer=storage__pb2.CreateStreamRequest.SerializeToString,
        response_deserializer=storage__pb2.CreateStreamResponse.FromString,
        )
    self.DeleteStream = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/DeleteStream',
        request_serializer=storage__pb2.DeleteStreamRequest.SerializeToString,
        response_deserializer=storage__pb2.DeleteStreamResponse.FromString,
        )
    self.GetStream = channel.unary_unary(
        '/bookkeeper.proto.storage.RootRangeService/GetStream',
        request_serializer=storage__pb2.GetStreamRequest.SerializeToString,
        response_deserializer=storage__pb2.GetStreamResponse.FromString,
        )


class RootRangeServiceServicer(object):
  """public service for metadata services
  """

  def CreateNamespace(self, request, context):
    """
    Namespace Methods

    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteNamespace(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetNamespace(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateStream(self, request, context):
    """
    Stream Methods

    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteStream(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetStream(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_RootRangeServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateNamespace': grpc.unary_unary_rpc_method_handler(
          servicer.CreateNamespace,
          request_deserializer=storage__pb2.CreateNamespaceRequest.FromString,
          response_serializer=storage__pb2.CreateNamespaceResponse.SerializeToString,
      ),
      'DeleteNamespace': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteNamespace,
          request_deserializer=storage__pb2.DeleteNamespaceRequest.FromString,
          response_serializer=storage__pb2.DeleteNamespaceResponse.SerializeToString,
      ),
      'GetNamespace': grpc.unary_unary_rpc_method_handler(
          servicer.GetNamespace,
          request_deserializer=storage__pb2.GetNamespaceRequest.FromString,
          response_serializer=storage__pb2.GetNamespaceResponse.SerializeToString,
      ),
      'CreateStream': grpc.unary_unary_rpc_method_handler(
          servicer.CreateStream,
          request_deserializer=storage__pb2.CreateStreamRequest.FromString,
          response_serializer=storage__pb2.CreateStreamResponse.SerializeToString,
      ),
      'DeleteStream': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteStream,
          request_deserializer=storage__pb2.DeleteStreamRequest.FromString,
          response_serializer=storage__pb2.DeleteStreamResponse.SerializeToString,
      ),
      'GetStream': grpc.unary_unary_rpc_method_handler(
          servicer.GetStream,
          request_deserializer=storage__pb2.GetStreamRequest.FromString,
          response_serializer=storage__pb2.GetStreamResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bookkeeper.proto.storage.RootRangeService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class StorageContainerServiceStub(object):
  """A general range server service
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetStorageContainerEndpoint = channel.unary_unary(
        '/bookkeeper.proto.storage.StorageContainerService/GetStorageContainerEndpoint',
        request_serializer=storage__pb2.GetStorageContainerEndpointRequest.SerializeToString,
        response_deserializer=storage__pb2.GetStorageContainerEndpointResponse.FromString,
        )


class StorageContainerServiceServicer(object):
  """A general range server service
  """

  def GetStorageContainerEndpoint(self, request, context):
    """Get the storage container endpoints
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_StorageContainerServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetStorageContainerEndpoint': grpc.unary_unary_rpc_method_handler(
          servicer.GetStorageContainerEndpoint,
          request_deserializer=storage__pb2.GetStorageContainerEndpointRequest.FromString,
          response_serializer=storage__pb2.GetStorageContainerEndpointResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bookkeeper.proto.storage.StorageContainerService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Exceptions raised by bookkeeper clients.
This module provides base classes for all errors raised by libraries based
on :mod:`bookkeeper.common`.
"""

from __future__ import absolute_import
from __future__ import unicode_literals
from bookkeeper.proto import storage_pb2

import six

try:
    import grpc
except ImportError:  # pragma: NO COVER
    grpc = None

# Lookup tables for mapping exceptions from gRPC transports.
# Populated by _APICallErrorMeta
_GRPC_CODE_TO_EXCEPTION = {}


class BKError(Exception):
    """Base class for all exceptions raised by bookkeeper Clients."""
    pass


@six.python_2_unicode_compatible
class RetryError(BKError):
    """Raised when a function has exhausted all of its available retries.
    Args:
        message (str): The exception message.
        cause (Exception): The last exception raised when retring the
            function.
    """
    def __init__(self, message, cause):
        super(RetryError, self).__init__(message)
        self.message = message
        self._cause = cause

    @property
    def cause(self):
        """The last exception raised when retrying the function."""
        return self._cause

    def __str__(self):
        return '{}, last exception: {}'.format(self.message, self.cause)


class _BKGRpcCallErrorMeta(type):
    """Metaclass for registering BKGRpcCallError subclasses."""
    def __new__(mcs, name, bases, class_dict):
        cls = type.__new__(mcs, name, bases, class_dict)
        if cls.grpc_status_code is not None:
            _GRPC_CODE_TO_EXCEPTION.setdefault(cls.grpc_status_code, cls)
        return cls


@six.python_2_unicode_compatible
@six.add_metaclass(_BKGRpcCallErrorMeta)
class BKGrpcCallError(BKError):
    """Base class for exceptions raised by calling API methods.
    Args:
        message (str): The exception message.
        errors (Sequence[Any]): An optional list of error details.
        response (Union[requests.Request, grpc.Call]): The response or
            gRPC call metadata.
    """

    grpc_status_code = None
    """Optional[grpc.StatusCode]: The gRPC status code associated with this
    error.
    This may be ``None`` if the exception does not match up to a gRPC error.
    """

    bk_status_code = None
    """Optional[bookkeeper.proto.StatusCode]: The bookkeeper storage status code
    associated with this error.
    This may be ```None` if the exception is a gRPC channel error.
    """

    def __init__(self, message, errors=(), response=None):
        super(BKGrpcCallError, self).__init__(message)
        self.message = message
        """str: The exception message."""
        self._errors = errors
        self._response = response

    def __str__(self):
        return 'grpc_status_code = {}, bk_status_code = {} : {}'\
            .format(self.grpc_status_code, self.bk_status_code, self.message)

    @property
    def errors(self):
        """Detailed error information.
        Returns:
            Sequence[Any]: A list of additional error details.
        """
        return list(self._errors)

    @property
    def response(self):
        """Optional[Union[requests.Request, grpc.Call]]: The response or
        gRPC call metadata."""
        return self._response


class ClientError(BKGrpcCallError):
    """Base class for all client error responses."""


class BadRequest(ClientError):
    """Exception mapping a ``400 Bad Request`` response."""
    code = storage_pb2.BAD_REQUEST
    grpc_status_code =\
        grpc.StatusCode.FAILED_PRECONDITION if grpc is not None else None


class IllegalOpError(ClientError):
    """Exception mapping a ``403 Illegal Op`` response."""
    code = storage_pb2.ILLEGAL_OP
    grpc_status_code =\
        grpc.StatusCode.FAILED_PRECONDITION if grpc is not None else None


class ServerError(BKGrpcCallError):
    """Base for 5xx responses."""


class StorageContainerNotFoundError(BKGrpcCallError):
    """Exception raised when storage container is not found"""
    code = None
    grpc_status_code =\
        grpc.StatusCode.NOT_FOUND if grpc is not None else None


class InternalServerError(ServerError):
    """Exception mapping a ``500 Internal Server Error`` response. or a
    :attr:`grpc.StatusCode.INTERNAL` error."""
    code = storage_pb2.INTERNAL_SERVER_ERROR
    grpc_status_code =\
        grpc.StatusCode.INTERNAL if grpc is not None else None


class UnimplementedError(ServerError):
    code = storage_pb2.NOT_IMPLEMENTED
    grpc_status_code =\
        grpc.StatusCode.UNIMPLEMENTED if grpc is not None else None


class UnexpectedError(BKGrpcCallError):
    code = storage_pb2.UNEXPECTED
    grpc_status_code =\
        grpc.StatusCode.UNKNOWN if grpc is not None else None


class StorageError(BKGrpcCallError):
    grpc_status_code = None


class FailureError(StorageError):
    code = storage_pb2.FAILURE


class BadVersionError(StorageError):
    code = storage_pb2.BAD_VERSION


class BadRevisionError(StorageError):
    code = storage_pb2.BAD_REVISION


class NamespaceError(StorageError):
    code = storage_pb2.UNEXPECTED


class InvalidNamespaceNameError(NamespaceError):
    code = storage_pb2.INVALID_NAMESPACE_NAME


class NamespaceNotFoundError(NamespaceError):
    code = storage_pb2.NAMESPACE_NOT_FOUND


class NamespaceExistsError(NamespaceError):
    code = storage_pb2.NAMESPACE_EXISTS


class StreamError(StorageError):
    code = storage_pb2.UNEXPECTED


class InvalidStreamNameError(StreamError):
    code = storage_pb2.INVALID_STREAM_NAME


class StreamNotFoundError(StreamError):
    code = storage_pb2.STREAM_NOT_FOUND


class StreamExistsError(StreamError):
    code = storage_pb2.STREAM_EXISTS


class TableError(StorageError):
    code = storage_pb2.UNEXPECTED


class InvalidKeyError(TableError):
    code = storage_pb2.INVALID_KEY


class KeyNotFoundError(TableError):
    code = storage_pb2.KEY_NOT_FOUND


class KeyExistsError(TableError):
    code = storage_pb2.KEY_EXISTS


_BK_CODE_TO_EXCEPTION_ = {
    storage_pb2.FAILURE: FailureError,
    storage_pb2.BAD_REQUEST: BadRequest,
    storage_pb2.ILLEGAL_OP: IllegalOpError,
    storage_pb2.INTERNAL_SERVER_ERROR: InternalServerError,
    storage_pb2.NOT_IMPLEMENTED: UnimplementedError,
    storage_pb2.UNEXPECTED: UnexpectedError,
    storage_pb2.BAD_VERSION: BadVersionError,
    storage_pb2.BAD_REVISION: BadRevisionError,
    storage_pb2.INVALID_NAMESPACE_NAME: InvalidNamespaceNameError,
    storage_pb2.NAMESPACE_EXISTS: NamespaceExistsError,
    storage_pb2.NAMESPACE_NOT_FOUND: NamespaceNotFoundError,
    storage_pb2.INVALID_STREAM_NAME: InvalidStreamNameError,
    storage_pb2.STREAM_EXISTS: StreamExistsError,
    storage_pb2.STREAM_NOT_FOUND: StreamNotFoundError,
    storage_pb2.INVALID_KEY: InvalidKeyError,
    storage_pb2.KEY_EXISTS: KeyExistsError,
    storage_pb2.KEY_NOT_FOUND: KeyNotFoundError
}


def exception_class_for_grpc_status(status_code):
    """Return the exception class for a specific :class:`grpc.StatusCode`.
    Args:
        status_code (grpc.StatusCode): The gRPC status code.
    Returns:
        :func:`type`: the appropriate subclass of :class:`BKGrpcCallError`.
    """
    return _GRPC_CODE_TO_EXCEPTION.get(status_code, BKGrpcCallError)


def from_grpc_status(status_code, message, **kwargs):
    """Create a :class:`BKGrpcCallError` from a :class:`grpc.StatusCode`.
    Args:
        status_code (grpc.StatusCode): The gRPC status code.
        message (str): The exception message.
        kwargs: Additional arguments passed to the :class:`BKGrpcCallError`
            constructor.
    Returns:
        BKGrpcCallError: An instance of the appropriate subclass of
            :class:`BKGrpcCallError`.
    """
    error_class = exception_class_for_grpc_status(status_code)
    error = error_class(message, **kwargs)

    if error.grpc_status_code is None:
        error.grpc_status_code = status_code

    return error


def from_grpc_error(rpc_exc):
    """Create a :class:`BKGrpcCallError` from a :class:`grpc.RpcError`.
    Args:
        rpc_exc (grpc.RpcError): The gRPC error.
    Returns:
        BKGrpcCallError: An instance of the appropriate subclass of
            :class:`BKGrpcError`.
    """
    if isinstance(rpc_exc, grpc.Call):
        return from_grpc_status(
            rpc_exc.code(),
            rpc_exc.details(),
            errors=(rpc_exc,),
            response=rpc_exc)
    elif isinstance(rpc_exc, grpc.RpcError):
        return from_grpc_error(
            rpc_exc.code(),
            rpc_exc.details(),
            errors=(rpc_exc,),
            response=rpc_exc
        )
    else:
        return BKGrpcCallError(
            str(rpc_exc), errors=(rpc_exc,), response=rpc_exc)


def exception_class_for_bk_status_code(status_code):
    return _BK_CODE_TO_EXCEPTION_.get(status_code, BKGrpcCallError)


def from_bk_status(status_code, message, **kwargs):
    error_class = exception_class_for_bk_status_code(status_code)
    error = error_class(message, **kwargs)

    if error.bk_status_code is None:
        error.bk_status_code = status_code

    return error


def from_table_rpc_response(rpc_resp):
    routing_header = rpc_resp.header
    status_code = routing_header.code
    if storage_pb2.SUCCESS == status_code:
        return rpc_resp
    else:
        raise from_bk_status(
            status_code,
            "",
            errors=(rpc_resp,),
            response=rpc_resp)


def from_root_range_rpc_response(rpc_resp):
    status_code = rpc_resp.code
    if storage_pb2.SUCCESS == status_code:
        return rpc_resp
    else:
        raise from_bk_status(
            status_code,
            "",
            errors=(rpc_resp,),
            response=rpc_resp)
